package CheckRelease::Plugin;

use strict;
use warnings;

use MT;
use base qw(MT::Plugin);

use vars qw( $VERSION $OLD_VERSION $SCHEMA_VERSION );
$VERSION = '1.2.02';
$OLD_VERSION = 0;
$SCHEMA_VERSION = 0.051; # Next of 0.051 is 0.053 caused asazoo

use Encode;
use CheckRelease::L10N;
use CustomFields::CheckRelease;
use CheckRelease::Task;
use CheckRelease::Util qw(get_param_session);
use CherryPick::LicenseManager;
use CherryPick::LicenseManager::CMS;

my $plugin = CheckRelease::Plugin->new({
	id => 'CheckRelease',
	key => __PACKAGE__,
	name => 'CheckRelease',
	l10n_class => 'CheckRelease::L10N',
	description => '<__trans phrase="Support your checking entries." />',
  doc_link => 'https://www.cherry-pick.jp/mt-plugins/checkrelease/',
  author_name => "cherry-pick Inc.",
  author_link => 'https://www.cherry-pick.jp/mt-plugins/checkrelease/',
	config_template => \&config_template,
	settings => new MT::PluginSettings([
		['checker_cross_check'],
		['select_checker'],
		['check_required'],
		['lock_entry'],
		['notify_link_type'],
		['edit_all_posts'],
		['use_ssl_url']
	]),
	schema_version => $SCHEMA_VERSION,
	version => $VERSION,
});

sub init_registry {
	my ($plugin) = @_;
	my $cfg = MT->config;
	$plugin->registry({
		config_settings => {
			'CRPreviewURLExpires' => { default => 60 * 60 * 24 * 14 },
			'CRRedirectURLExpires' => { default => 60 * 5 },
		},
		object_types => {
			'entry' => {
				'checked_on' => 'datetime',
				'checked_by' => 'integer(11)',
				'check_status' => 'smallint(6)',
				'check_requested_authors' => 'text',
				'check_note' => 'text',
			},
		},
		default_templates => {
			base_path => 'templates/global',
			'global:email' => {
				return_to_entry_email => {
					label => 'Return to notification'
				},
			},
		},
		tags => {
			function => {
				CheckNote => '$CheckRelease::CheckRelease::Template::Tags::Entry::_hdlr_return_note',
			},
		},
		tasks => {
			'RevisionFuturePost' => {
				key => 'RevisionFuturePost',
				label => "Publish Scheduled Entries's Revision",
				frequency => $cfg->FuturePostFrequency * 60,
				code => sub {
					CheckRelease::Task->future_post;
				},
			},
		},
		applications => {
            cms => {
                methods => {
                  checkrelease_register_license_manager => sub {
                    CherryPick::LicenseManager::CMS::handle_register_licensekey(CheckRelease::Plugin->instance->license_manager, @_);
                  },
                },
                callbacks => {
                	'template_param.dashboard' => \&template_param_dashboard,
                	'init_request' => {
                		priority => 1,
                		code => \&on_init_request,
                	},
                  'template_param.cfg_plugin' => sub {
                    CherryPick::LicenseManager::CMS::handle_template_param_cfg_plugin(CheckRelease::Plugin->instance->license_manager, @_);
                  },
                },
            },
        },
		list_properties => {
			entry => {
				title => {
					html => '$CheckRelease::CheckRelease::ListProperty::_hdlr_entry_title_html',
				},
				cr_writer_revision => {
					base => '__common.string',
					label => sub { MT->component('CheckRelease')->translate('Revision'); },
					sort => 0,
					html => '$CheckRelease::CheckRelease::ListProperty::_hdlr_entry_revision_html',
					condition => '$CheckRelease::CheckRelease::ListProperty::_hdlr_condition',
					order => 210,
					default => 'default',
				},
			},
		},
	});
}

sub instance {
	$plugin;
}

sub license_manager {
    my $plugin = shift;

    my $local_active_env_name = $plugin->name .'LocalActivation';
    my $lm = CherryPick::LicenseManager->new({
        plugin          => $plugin,
        register_url    => 'https://mtlmc.cherry-pick.jp/api/licenses/register',
        confirm_url     => 'https://mtlmc.cherry-pick.jp/api/licenses/register',
        confirm_interval => 14,
        trial_days => 45,
        local_activation => MT->config->$local_active_env_name ? 1 : 0,
    }) || return $plugin->error(CherryPick::LicenseManager->errstr);

    $lm;
}

sub config_template {
	my ($plugin, $param, $scope) = @_;

	# スコープを確認
	my $blog_id = 0;
	my $blog;
	unless ( $scope eq 'system' ) {
		($blog_id) = ($scope =~ /blog:(.+)$/);
		die $plugin->translate("Can't find blog_id.") unless $blog_id;
		require MT::App::CMS;
		my $app = MT::App::CMS->instance;
		$blog = $app->model('blog')->load($blog_id);
		die $plugin->translate("Can't find such a blog.") unless ref $blog;
		return '' if MT->version_number < 6.0 && !$blog->parent_id;
	}
	$param->{blog_id} = $blog_id;

	my $tmpl =<< 'EOT';
<input type="hidden" name="edit_all_posts" value="edit_all_posts" />
<mtapp:setting id="select_checker" label="<__trans phrase="Selectable or Unselectable" />">
	<p><__trans phrase="Check 'Selectable', when You want to that writers can select a certificater."></p>
	<input type="radio" name="select_checker" value="1"<mt:if name="select_checker"> checked="checked"</mt:if> />&nbsp;<__trans phrase="Selectable" />
	&nbsp;&nbsp;
	<input type="radio" name="select_checker" value="0"<mt:unless name="select_checker"> checked="checked"</mt:unless> />&nbsp;<__trans phrase="Unselectable" />
</mtapp:setting>
<mtapp:setting id="check_required" label="<__trans phrase="Required check" />">
	<p><__trans phrase="Select 'Required', if check is required. This setting is available in 'Selectable'."></p>
	<input type="radio" name="check_required" value="1"<mt:if name="check_required"> checked="checked"</mt:if> />&nbsp;<__trans phrase="Required" />
	&nbsp;&nbsp;
	<input type="radio" name="check_required" value="0"<mt:unless name="check_required"> checked="checked"</mt:unless> />&nbsp;<__trans phrase="Not required" />
</mtapp:setting>
<mtapp:setting id="lock_entry" label="<__trans phrase="Lock entry or not" />">
	<p><__trans phrase="Select which you want to lock entries which were written by writers, and are subscribing to be approved by certificater."></p>
	<input type="radio" name="lock_entry" value="1"<mt:if name="lock_entry"> checked="checked"</mt:if> />&nbsp;<__trans phrase="Lock" />
	&nbsp;&nbsp;
	<input type="radio" name="lock_entry" value="0"<mt:unless name="lock_entry"> checked="checked"</mt:unless> />&nbsp;<__trans phrase="Unlock" />
</mtapp:setting>
<mtapp:setting id="checker_cross_check" label="<__trans phrase="Cross check" />">
	<p><__trans phrase="Select which checkers are checked each other."></p>
	<input type="radio" name="checker_cross_check" value="0"<mt:unless name="checker_cross_check"> checked="checked"</mt:unless> />&nbsp;<__trans phrase="Not request" />
	&nbsp;&nbsp;
	<input type="radio" name="checker_cross_check" value="1"<mt:if name="checker_cross_check"> checked="checked"</mt:if> />&nbsp;<__trans phrase="Request" />
</mtapp:setting>
EOT

	$tmpl .= << 'EOT' if $scope eq 'system';
<mtapp:setting id="use_ssl_url" label="<__trans phrase="Management URLs" />">
	<p>
		<__trans phrase="Do you want to replace URL schema in notification emails to SSL(https://) ?"><br />
		<span style="color:red"><__trans phrase="Movable Type cloud maybe need this option."></span>
	</p>
	<input type="radio" name="use_ssl_url" value="1"<mt:if name="use_ssl_url"> checked="checked"</mt:if> />&nbsp;<__trans phrase="Use SSL" />
	&nbsp;&nbsp;
	<input type="radio" name="use_ssl_url" value="0"<mt:unless name="use_ssl_url"> checked="checked"</mt:unless> />&nbsp;<__trans phrase="Not replace" />
</mtapp:setting>
EOT

	my $html = '';

    my $lm = $plugin->license_manager
        || return $plugin->errstr;

	if ( $scope eq 'system' ) {
		# ライセンスキー登録用テンプレート

	    $param->{email} = $lm->email;
	    $param->{licensekey} = $lm->licensekey;
	    $param->{is_registered} = $lm->is_registered ? 1 : 0;
	    $param->{is_expired} = $lm->is_expired ? 1 : 0;
	    $param->{remainder_days} = $lm->remainder_days;

	    my $lm_tmpl = <<'EOT';
	    <mt:if name="is_expired">
	        <p><__trans phrase="Trial is expired."></p>
	    <mt:else>
	        <mt:unless name="is_registered">
	            <mtapp:setting
	                id="probation"
	                label="<__trans phrase="Probation:">"
	                hint="<__trans phrase="Hint">">
	                <__trans phrase="Ends in"> <mt:var name="remainder_days"> <__trans phrase="days">
	            </mtapp:setting>
	        </mt:unless>
	    </mt:if>
	    <mtapp:setting
	        id="license-key-pref"
	        label="<__trans phrase="License key:">"
	        hint="<__trans phrase="Hint">">
	        <input type="text" name="licensekey" id="license-key" value="<mt:var name="licensekey" escape="html">" />
	    </mtapp:setting>
	    <mtapp:setting
	        id="email-pref"
	        label="<__trans phrase="E-mail:">"
	        hint="<__trans phrase="Hint">">
	        <input type="text" name="email" id="email" value="<mt:var name="email" escape="html">" />
	    </mtapp:setting>
	    <div class="actions-bar settings-actions-bar">
	        <button
	            mt:mode="save_plugin_config"
	            class="save action primary-button"
	            onclick="this.form['__mode'].value = 'checkrelease_register_license_manager'; this.form.submit(); return false;"
	            ><__trans phrase="Register license"></button>
	    </div>
EOT

		$html = $lm_tmpl;
		$html .= $tmpl unless $param->{is_expired};
	}
	else {
		return '' if !$lm->is_registered && $lm->is_expired;
		$html .= $tmpl;
	}
	$html;
}

sub on_init_request {
	my ( $cb, $app ) = @_;
	my $token = $app->param('cr_token') || return 1;
	my $salt = $app->param('cr_salt');

	defined($token) && length($token) == 32 || return;
	defined($salt) && ($salt =~ m!^\d+$! && $salt <= 1000000) || return 1;

	my $hash = get_param_session($token, $salt);

	unless ( $hash ) {
		$app->mode('dashboard');
		$app->param('__mode', 'dashboard');
		$app->param('cr_error', Encode::encode_utf8($plugin->translate('This URL is invalid or expired.')));
		return 1;
	}

	my %param = $app->param_hash;
	foreach my $key ( keys %$hash ) {
		next if defined $param{$key};
		my $value = $hash->{$key};
		next unless defined $value;
		$app->param($key, Encode::encode_utf8($value));
	}

	1;
}

sub template_param_dashboard {
	my ( $cb, $app, $param, $tmpl ) = @_;

	my $node = $tmpl->createElement('setvarblock', { name => 'system_msg', append => 1 });
	$node->innerHTML(q{
<mt:if name="cr_error">
  <mtapp:statusmsg
     id="checkrelease-error"
     class="error"
     can_close="0">
    <mt:var name="cr_error">
  </mtapp:statusmsg>
</mt:if>
	});

	$param->{cr_error} = $app->param('cr_error');
	$tmpl->insertBefore($node, $tmpl->getElementById('header_include'));

	1;
}

1;
